(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[5405],{34376:function(n,u,e){n.exports=e(8641)},63878:function(n,u,e){"use strict";e.r(u);var t=e(82684),c=e(34376);u.default=function(){var n=(0,c.useRouter)();(0,t.useEffect)((function(){n.replace("/pipelines")}),[])}},48312:function(n,u,e){(window.__NEXT_P=window.__NEXT_P||[]).push(["/",function(){return e(63878)}])}},function(n){n.O(0,[9774,2888,179],(function(){return u=48312,n(n.s=u);var u}));var u=n.O();_N_E=u}]);