"use strict";(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[9633],{9633:function(e){e.exports=JSON.parse('{"base":"vs-dark","inherit":true,"rules":[{"background":"0B0A09","token":""},{"foreground":"d27518c2","token":"constant"},{"foreground":"a89100b5","token":"entity"},{"foreground":"ba6912","token":"entity.other"},{"foreground":"949c8b","token":"keyword"},{"foreground":"ffee80","token":"storage"},{"foreground":"c7a4a1b5","token":"string -string.unquoted.old-plist -string.unquoted.heredoc"},{"foreground":"c7a4a1b5","token":"string.unquoted.heredoc string"},{"foreground":"706d5b","fontStyle":"italic","token":"comment"},{"foreground":"9fc28a","token":"support"},{"foreground":"d1a796","token":"variable"},{"foreground":"ff80e1","token":"variable.language"},{"foreground":"ffee80","token":"meta.function-call"},{"foreground":"f8f8f8","background":"a41300","token":"invalid"},{"foreground":"d9d59f","background":"24231d4d","token":"text source"},{"foreground":"d9d59f","background":"24231d4d","token":"string.unquoted.heredoc"},{"foreground":"d9d59f","background":"24231d4d","token":"source source"},{"foreground":"7efcff","token":"entity.other.inherited-class"},{"foreground":"439740ba","token":"string.quoted source"},{"foreground":"60db5dba","token":"string constant"},{"foreground":"7dffc0a6","token":"string.regexp"},{"foreground":"b8b960","token":"string variable"},{"foreground":"85873a","token":"support.function"},{"foreground":"c27e66","token":"support.constant"},{"foreground":"ff1e00","token":"support.class.exception"},{"foreground":"8996a8","token":"meta.preprocessor.c"},{"foreground":"afc4db","token":"meta.preprocessor.c keyword"},{"foreground":"73817d","token":"meta.sgml.html meta.doctype"},{"foreground":"73817d","token":"meta.sgml.html meta.doctype entity"},{"foreground":"73817d","token":"meta.sgml.html meta.doctype string"},{"foreground":"73817d","token":"meta.xml-processing"},{"foreground":"73817d","token":"meta.xml-processing entity"},{"foreground":"73817d","token":"meta.xml-processing string"},{"foreground":"babd9c","token":"meta.tag"},{"foreground":"babd9c","token":"meta.tag entity"},{"foreground":"99a190","token":"meta.selector.css entity.name.tag"},{"foreground":"cc8844","token":"meta.selector.css entity.other.attribute-name.id"},{"foreground":"cfb958","token":"meta.selector.css entity.other.attribute-name.class"},{"foreground":"e0ddad","token":"support.type.property-name.css"},{"foreground":"aeb14b","token":"meta.property-group support.constant.property-value.css"},{"foreground":"aeb14b","token":"meta.property-value support.constant.property-value.css"},{"foreground":"ffb010","token":"meta.preprocessor.at-rule keyword.control.at-rule"},{"foreground":"999179","token":"meta.property-value support.constant.named-color.css"},{"foreground":"999179","token":"meta.property-value constant"},{"foreground":"eb939a","token":"meta.constructor.argument.css"},{"foreground":"f8f8f8","background":"000e1a","token":"meta.diff"},{"foreground":"f8f8f8","background":"000e1a","token":"meta.diff.header"},{"foreground":"f8f8f8","background":"800f00","token":"markup.deleted"},{"foreground":"f8f8f8","background":"806f00","token":"markup.changed"},{"foreground":"f8f8f8","background":"228000","token":"markup.inserted"},{"background":"8fddf630","token":"markup.raw"},{"background":"005baa","token":"markup.quote"},{"background":"0f0040","token":"markup.list"},{"foreground":"9d80ff","fontStyle":"bold","token":"markup.bold"},{"foreground":"80ffbb","fontStyle":"italic","token":"markup.italic"},{"fontStyle":"bold","token":"markup.heading"}],"colors":{"editor.foreground":"#FCFFE0","editor.background":"#0B0A09","editor.selectionBackground":"#AA00FF73","editor.lineHighlightBackground":"#38403D","editorCursor.foreground":"#FF9900","editorWhitespace.foreground":"#FFB16F52"}}')}}]);