"use strict";(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[5729],{15729:function(o){o.exports=JSON.parse('{"base":"vs-dark","inherit":true,"rules":[{"background":"222C28","token":""},{"foreground":"666c68","background":"151c19","token":"comment"},{"foreground":"c23b00","token":"storage"},{"foreground":"c23b00","token":"support.type"},{"foreground":"ffffff","background":"151c19","token":"string.unquoted.embedded"},{"foreground":"ffffff","background":"151c19","token":"text source"},{"foreground":"ffffff","background":"151c19","token":"string.unquoted"},{"foreground":"e9470000","background":"1a0700","token":"constant.character.escaped"},{"foreground":"e9470000","background":"1a0700","token":"string source - string.unquoted.embedded"},{"foreground":"e9470000","background":"1a0700","token":"string string source"},{"foreground":"c23800","background":"1a0700","token":"string - string source"},{"foreground":"c23800","background":"1a0700","token":"string source string"},{"foreground":"c23800","background":"1a0700","token":"meta.scope.heredoc"},{"foreground":"e98800","token":"constant.numeric"},{"foreground":"648bd2","token":"variable.language"},{"foreground":"648bd2","token":"variable.other"},{"foreground":"e98800","token":"constant"},{"foreground":"a8b3ab","background":"161d1a","token":"other.preprocessor"},{"foreground":"a8b3ab","background":"161d1a","token":"entity.name.preprocessor"},{"foreground":"a8b3ab","token":"entity.name.function"},{"foreground":"a8b3ab","token":"keyword.operator"},{"foreground":"a8b3ab","token":"keyword.other.name-of-parameter"},{"foreground":"9a2f00","token":"entity.name.class"},{"foreground":"648bd2","token":"variable.parameter"},{"foreground":"666c68","token":"storage.type.method"},{"foreground":"a39e64","token":"keyword"},{"foreground":"a39e64","token":"storage.type.function.php"},{"foreground":"ffffff","background":"990000ad","token":"invalid"},{"foreground":"000000","background":"ffd0d0","token":"invalid.trailing-whitespace"},{"foreground":"588e60","token":"support.function"},{"foreground":"5778b6","token":"support.class"},{"foreground":"5778b6","token":"support.type"},{"foreground":"5778b6","token":"entity.name"},{"foreground":"c87500","token":"support.constant"},{"foreground":"5879b7","token":"support.other.variable"},{"foreground":"68685b","token":"declaration.xml-processing"},{"foreground":"888888","token":"declaration.doctype"},{"foreground":"888888","token":"declaration.doctype.DTD"},{"foreground":"a65eff","token":"declaration.tag"},{"foreground":"a65eff","token":"entity.name.tag"},{"foreground":"909993","token":"entity.other.attribute-name"},{"foreground":"90999380","token":"punctuation"},{"foreground":"7642b7","token":"entity.other.inherited-class"},{"foreground":"ffffff","background":"00000059","token":"meta.scope.changed-files.svn"},{"foreground":"ffffff","background":"00000059","token":"markup.inserted.svn"},{"foreground":"ffffff","background":"00000059","token":"markup.changed.svn"},{"foreground":"ffffff","background":"00000059","token":"markup.deleted.svn"},{"background":"78807b0a","token":"meta.section"},{"background":"78807b0a","token":"meta.section meta.section"},{"background":"78807b0a","token":"meta.section meta.section meta.section"}],"colors":{"editor.foreground":"#FFFFFF","editor.background":"#222C28","editor.selectionBackground":"#91999466","editor.lineHighlightBackground":"#0C0D0C40","editorCursor.foreground":"#FFFFFF","editorWhitespace.foreground":"#666C6880"}}')}}]);