(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[1009],{66757:function(n,e,t){"use strict";var i=t(19319),u=t(94178),r=t(67240),o="localStorage"in t.g&&t.g.localStorage?t.g.localStorage:i;function c(n,e){return 1===arguments.length?s(n):l(n,e)}function s(n){const e=o.getItem(n);return u(e)}function l(n,e){try{return o.setItem(n,JSON.stringify(e)),!0}catch(t){return!1}}c.set=l,c.get=s,c.remove=function(n){return o.removeItem(n)},c.clear=function(){return o.clear()},c.backend=function(n){return n&&(o=n),o},c.on=r.on,c.off=r.off,n.exports=c},94178:function(n){"use strict";n.exports=function(n){const e=function(n){try{return JSON.parse(n)}catch(e){return n}}(n);return void 0===e?null:e}},19319:function(n){"use strict";var e={};n.exports={getItem:function(n){return n in e?e[n]:null},setItem:function(n,t){return e[n]=t,!0},removeItem:function(n){return!!(n in e)&&delete e[n]},clear:function(){return e={},!0}}},67240:function(n,e,t){"use strict";var i=t(94178),u={};function r(n){n||(n=t.g.event);var e=u[n.key];e&&e.forEach((function(e){e(i(n.newValue),i(n.oldValue),n.url||n.uri)}))}n.exports={on:function(n,e){u[n]?u[n].push(e):u[n]=[e],t.g.addEventListener?t.g.addEventListener("storage",r,!1):t.g.attachEvent?t.g.attachEvent("onstorage",r):t.g.onstorage=r},off:function(n,e){var t=u[n];t.length>1?t.splice(t.indexOf(e),1):u[n]=[]}}},66050:function(n,e,t){"use strict";var i;t.d(e,{V:function(){return i}}),function(n){n.CANCELLED="cancelled",n.COMPLETED="completed",n.FAILED="failed",n.INITIAL="initial",n.RUNNING="running"}(i||(i={}))},47409:function(n,e,t){"use strict";t.d(e,{V:function(){return i}});var i=t(66050).V},92597:function(n,e,t){"use strict";t.r(e);var i=t(12691),u=t.n(i),r=t(34376),o=t.n(r),c=t(82684),s=t(60328),l=t(48933),a=t(10919),d=t(47409),f=t(17903),p=t(19711),g=t(41374),v=t(10503),h=t(49125),_=t(24224),m=t(28598);e.default=function(){var n=g.ZP.pipelines.list().data,e=g.ZP.pipeline_runs.list().data,t=(0,c.useMemo)((function(){return(null===n||void 0===n?void 0:n.pipelines)||[]}),[n]),i=(0,c.useMemo)((function(){return(0,_.HK)(t,(function(n){return n.uuid}))}),[t]),r=(0,c.useMemo)((function(){return(null===e||void 0===e?void 0:e.pipeline_runs)||[]}),[e]);return(0,m.jsx)(l.Z,{title:"Pipeline runs",uuid:"pipeline_runs/index",children:(0,m.jsx)(f.Z,{columnFlex:[null,1,1,1,null,null],columns:[{uuid:"Date"},{uuid:"Status"},{uuid:"Pipeline"},{uuid:"Trigger"},{uuid:"Block runs"},{uuid:"Logs"}],rows:r.map((function(n){var e,t=n.block_runs_count,r=n.created_at,c=n.id,l=n.pipeline_schedule_id,f=n.pipeline_schedule_name,g=n.pipeline_uuid,_=n.status;return[(0,m.jsx)(p.ZP,{default:!0,monospace:!0,children:r}),(0,m.jsx)(p.ZP,{danger:d.V.FAILED===_,info:d.V.INITIAL===_,monospace:!0,muted:d.V.CANCELLED===_,success:d.V.COMPLETED===_,warning:d.V.RUNNING===_,children:_}),(0,m.jsx)(u(),{as:"/pipelines/".concat(g),href:"/pipelines/[pipeline]",passHref:!0,children:(0,m.jsx)(a.Z,{bold:!0,sameColorAsText:!0,children:null===(e=i[g])||void 0===e?void 0:e.name})}),(0,m.jsx)(u(),{as:"/pipelines/".concat(g,"/triggers/").concat(l),href:"/pipelines/[pipeline]/triggers/[...slug]",passHref:!0,children:(0,m.jsx)(a.Z,{bold:!0,sameColorAsText:!0,children:f})}),(0,m.jsx)(p.ZP,{default:!0,monospace:!0,children:t}),(0,m.jsx)(s.Z,{default:!0,iconOnly:!0,noBackground:!0,onClick:function(){return o().push("/pipelines/".concat(g,"/logs?pipeline_run_id[]=").concat(c))},children:(0,m.jsx)(v.B4,{default:!0,size:2*h.iI})})]}))})})}},60710:function(n,e,t){(window.__NEXT_P=window.__NEXT_P||[]).push(["/pipeline-runs",function(){return t(92597)}])}},function(n){n.O(0,[5339,2874,3903,7578,9774,2888,179],(function(){return e=60710,n(n.s=e);var e}));var e=n.O();_N_E=e}]);