self.__BUILD_MANIFEST=function(s,e,i,c,p,t,a,n,l,d,u,f,r,g,k,h,j,b){return{__rewrites:{beforeFiles:[],afterFiles:[{source:"/CLOUD_NOTEBOOK_BASE_PATH_PLACEHOLDER_/datasets",destination:u},{source:"/CLOUD_NOTEBOOK_BASE_PATH_PLACEHOLDER_",destination:u}],fallback:[]},"/":["static/chunks/pages/index-329878ea2836950a.js"],"/_error":["static/chunks/pages/_error-235304e5badb19eb.js"],"/datasets":[s,i,e,t,g,"static/chunks/pages/datasets-8074498c4af1361e.js"],"/datasets/[...slug]":[k,"static/chunks/8a28b14e-b845f39223877713.js",s,i,a,n,h,"static/chunks/17-6f5e28da6ea3bc27.js",e,t,l,f,j,g,"static/chunks/pages/datasets/[...slug]-85e7978c070c218d.js"],"/pipeline-runs":[s,e,c,p,"static/chunks/pages/pipeline-runs-fee8dcb7eeb43713.js"],"/pipelines":[s,i,e,c,p,"static/chunks/pages/pipelines-932471d222c503ba.js"],"/pipelines/[pipeline]":["static/chunks/pages/pipelines/[pipeline]-0c6dc5356df3d124.js"],"/pipelines/[pipeline]/edit":[k,s,i,a,n,r,h,"static/chunks/2437-52e7a8f62f271f5b.js",e,c,t,l,d,f,j,"static/chunks/pages/pipelines/[pipeline]/edit-830978393f0b3d6f.js"],"/pipelines/[pipeline]/logs":[s,i,e,c,p,"static/chunks/pages/pipelines/[pipeline]/logs-e2211fcac0f94043.js"],"/pipelines/[pipeline]/runs":[s,i,a,r,e,c,p,d,b,"static/chunks/pages/pipelines/[pipeline]/runs-471e8b66b1527a05.js"],"/pipelines/[pipeline]/triggers":[s,i,a,n,e,c,p,t,l,d,"static/chunks/pages/pipelines/[pipeline]/triggers-a21820f4e00ad9fd.js"],"/pipelines/[pipeline]/triggers/[...slug]":[s,i,a,n,r,"static/chunks/4495-aaa900f908a8a255.js",e,c,p,t,l,d,f,b,"static/chunks/pages/pipelines/[pipeline]/triggers/[...slug]-96961f37335626eb.js"],sortedPages:["/","/_app","/_error",u,"/datasets/[...slug]","/pipeline-runs","/pipelines","/pipelines/[pipeline]","/pipelines/[pipeline]/edit","/pipelines/[pipeline]/logs","/pipelines/[pipeline]/runs","/pipelines/[pipeline]/triggers","/pipelines/[pipeline]/triggers/[...slug]"]}}("static/chunks/5339-11f4757850474424.js","static/chunks/2874-bb1ac5b9276d832d.js","static/chunks/2083-31bedbe3b4490815.js","static/chunks/3903-5947ce1f75a8a1ec.js","static/chunks/7578-bbf89628b90c5c0c.js","static/chunks/5981-6794e9c8c762e352.js","static/chunks/2860-16f4089b788792f5.js","static/chunks/7875-4b4f93a0a59aec48.js","static/chunks/5584-66a6dd43d2a6a4ca.js","static/chunks/6532-81e6585ef7faa551.js","/datasets","static/chunks/91-322e26ca10302cce.js","static/chunks/7619-11107d4d093f5d51.js","static/chunks/3887-5c8404eec6532f49.js","static/chunks/29107295-989a0767a635d9d5.js","static/chunks/9735-f04138e770fcb18b.js","static/chunks/9916-97b0556d0e96e8b6.js","static/chunks/4953-fa7d224a8038915c.js"),self.__BUILD_MANIFEST_CB&&self.__BUILD_MANIFEST_CB();